from urlmagic.core import MemberUrlGenerator
from django.contrib.auth.decorators import user_passes_test


class SuperuserUrlGenerator(MemberUrlGenerator):

    @classmethod
    def adjust_dict(cls, model, d):
        d.setdefault("format_kwargs", {})
        d["format_kwargs"].setdefault("role", "superuser")
        super(SuperuserUrlGenerator, cls).adjust_dict(model, d)
        return d

    @classmethod
    def adjust_result(cls, r):
        super(SuperuserUrlGenerator, cls).adjust_result(r)
        r._callback = user_passes_test(lambda u: u.is_superuser)(r._callback)
        return r



from urlmagic import mixins
from urlmagic.utils import get_user_field_names
from urlmagic.views import filtered


class MyMixin(object):
    def __init__(self, *args, **kwargs):
        super(MyMixin, self).__init__(*args, **kwargs)
        for field_name in get_user_field_names(self.model):
            self.queryset_filter.setdefault(field_name, filtered.FilteredViewMixin.REQUEST_USER)


class MyListView(MyMixin, filtered.FilteredListView):
    pass


class MyDetailView(MyMixin, filtered.FilteredDetailView):
    pass


class MyCreateView(MyMixin, filtered.FilteredCreateView):
    pass


class MyUpdateView(MyMixin, filtered.FilteredUpdateView):
    pass


class MyDeleteView(MyMixin, filtered.FilteredDeleteView):
    pass



from urlmagic.core import UrlGenerator
from django.contrib.auth.decorators import login_required


class MemberUrlGenerator(UrlGenerator):

    @classmethod
    def adjust_dict(cls, model, d):
        d.setdefault("format_kwargs", {})
        d["format_kwargs"].setdefault("role", "member")
        super(MemberUrlGenerator, cls).adjust_dict(model, d)
        return d

    @classmethod
    def adjust_result(cls, r):
        super(MemberUrlGenerator, cls).adjust_result(r)
        r._callback = login_required(r._callback)
        return r



from urlmagic.member import MemberUrlGenerator
from urlmagic.views import my


class MyUrlGenerator(MemberUrlGenerator):

    default_views = {
        "add": my.MyCreateView,
        "delete": my.MyDeleteView,
        "detail": my.MyDetailView,
        "edit": my.MyUpdateView,
        "list": my.MyListView,
    }

    @classmethod
    def adjust_dict(cls, model, d):
        d.setdefault("format_kwargs", {})
        d["format_kwargs"].setdefault("role", "my")
        super(MyUrlGenerator, cls).adjust_dict(model, d)
        return d



from django.contrib.auth.models import User
from django.db.models.fields.related import ForeignKey
from django.utils.text import slugify


def model_names(model):
    names = {}
    names["model_system"] = unicode(model._meta.module_name)
    names["model_singular"] = unicode(model._meta.verbose_name)
    names["model_singular_slug"] = unicode(slugify(names["model_singular"]))
    names["model_singular_short"] = names["model_singular_slug"].replace("-", "")
    names["model_plural"] = unicode(model._meta.verbose_name_plural)
    names["model_plural_slug"] = unicode(slugify(names["model_plural"]))
    names["model_plural_short"] = names["model_plural_slug"].replace("-", "")
    names["model_module"] = unicode(model.__module__.split(".")[-2])
    return names


def get_user_field_names(model):
    return [
        field.name
        for field in model._meta.fields
        if isinstance(field, ForeignKey) and field.related.parent_model is User
    ]



from urlmagic import mixins
from urlmagic.utils import get_user_field_names
from urlmagic.views import core


class FilteredViewMixin(object):
    queryset_filter = {}
    REQUEST_USER = "request.user"

    def get_queryset(self):
        if self.REQUEST_USER in self.queryset_filter.values() and not self.request.user.is_authenticated():
            raise PermissionDenied
        return super(FilteredViewMixin, self).get_queryset().filter(**dict(
            (field_slug, self.request.user if url_variable == self.REQUEST_USER else self.kwargs[url_variable])
            for field_slug, url_variable in self.queryset_filter.iteritems()
        ))


class FilteredListView(FilteredViewMixin, core.ContextListView):
    pass


class FilteredDetailView(FilteredViewMixin, mixins.SingularViewMixin, core.ContextDetailView):
    pass


class FilteredCreateView(FilteredViewMixin, mixins.FormRequestViewMixin, core.ContextCreateView):
    pass


class FilteredUpdateView(FilteredViewMixin, mixins.SingularViewMixin, mixins.FormRequestViewMixin, core.ContextUpdateView):
    pass


class FilteredDeleteView(FilteredViewMixin, mixins.SingularViewMixin, core.ContextDeleteView):
    pass



from distutils.core import setup

setup(
    name='django-urlmagic',
    version='0.3.9',
    packages=['urlmagic', 'urlmagic.views'],
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    long_description=open('README.md').read(),
    url='https://github.com/bradbeattie/django-urlmagic',
    author='Brad Beattie',
    author_email='bradbeattie@gmail com',
)



from urlmagic.utils import get_user_field_names
from django.core.exceptions import PermissionDenied


class ContextViewMixin(object):
    extra_context = {}

    def get_context_data(self, *args, **kwargs):
        context = super(ContextViewMixin, self).get_context_data(*args, **kwargs)
        context.update(self.extra_context)
        return context


class AutomaticUserFormMixin(object):
    def save(self, *args, **kwargs):
        for field_name in getattr(self, "user_fields", get_user_field_names(self.instance)):
            setattr(self.instance, field_name, self.request.user)
        return super(AutomaticUserFormMixin, self).save(*args, **kwargs)


class SingularViewMixin(object):
    def adjust_kwargs(self):
        pk = self.kwargs.get(self.pk_url_kwarg, None)
        slug = self.kwargs.get(self.slug_url_kwarg, None)
        if pk is None and slug is None:
            queryset = self.get_queryset()
            if queryset.count():
                self.kwargs["pk"] = queryset[0].pk

    def get(self, request, *args, **kwargs):
        self.adjust_kwargs()
        return super(SingularViewMixin, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.adjust_kwargs()
        return super(SingularViewMixin, self).post(request, *args, **kwargs)


class FormRequestViewMixin(object):
    def get_form(self, form_class):
        form = super(FormRequestViewMixin, self).get_form(form_class)
        form.request = self.request
        return form



from django.views.generic.detail import DetailView
from django.views.generic.edit import UpdateView, CreateView, DeleteView
from django.views.generic.list import ListView
from urlmagic import mixins


class ContextListView(mixins.ContextViewMixin, ListView):
    pass


class ContextDetailView(mixins.ContextViewMixin, DetailView):
    pass


class ContextCreateView(mixins.ContextViewMixin, CreateView):
    pass


class ContextUpdateView(mixins.ContextViewMixin, UpdateView):
    pass


class ContextDeleteView(mixins.ContextViewMixin, DeleteView):
    pass
